#include <stdlib.h>
#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>  /* sei() */
#include <util/delay.h>     /* _delay_ms() */
#include <avr/eeprom.h>

#include <avr/pgmspace.h>   /* usbdrv.h */
#include "usbdrv/usbdrv.h"


#include "i2c.c"
#include "lm75.c"
#define LM75_ADDR       (LM75_ADDR_BASE + 0) //pasitikrinti



/* ------------------------------------------------------------------------- */
/* ------------------------------------------------------------------------- */

PROGMEM char usbHidReportDescriptor[22] = {    /* USB report */
    0x06, 0x00, 0xff,              // USAGE_PAGE (Generic Desktop)
    0x09, 0x01,                    // USAGE (Vendor Usage 1)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x26, 0xff, 0x00,              //   LOGICAL_MAXIMUM (255)
    0x75, 0x08,                    //   REPORT_SIZE (8)
    0x95, 0x80,                    //   REPORT_COUNT (128)
    0x09, 0x00,                    //   USAGE (Undefined)
    0xb2, 0x02, 0x01,              //   FEATURE (Data,Var,Abs,Buf)
    0xc0                           // END_COLLECTION
};

static uchar    currentAddress;
static uchar    bytesRemaining;
static uchar	tempa1;
static uchar	tempa2;

/* ------------------------------------------------------------------------- */

void ledof(void) { PORTB |=1 << 3; }
void ledon(void) { PORTB &= ~(1 << 3); }

int sensors_init(void)
{
	i2c_init();
	return lm75_configure(LM75_ADDR, 0x60); //cia komanda FM75, daugiau tikslumo. Skaityti FM75/LM75 datasheeta.
}


void readtemp(void)
{

    unsigned char dst[2];
	int x=0;

dst[0]=0; dst[1]=0;
		// temperaturos nuskaitymas is FM75 cipo 

			x = lm75_readRegister(LM75_ADDR, LM75_REG_TEMP, &dst[0], 2);
			if (x==0){
				tempa1=dst[0];
				tempa2=dst[1];
			} else {
				ledon();      /* LED on */
				sensors_init();
				wdt_reset();
				_delay_ms(1);
				wdt_reset();
					}
			

}


uchar   usbFunctionRead(uchar *data, uchar len)
{
uchar i;
ledon();      /* LED on */
for(i=0;i<8;i++){ data[i]=i;}
    if(len > bytesRemaining) len = bytesRemaining;
    //eeprom_read_block(data, (uchar *)0 + currentAddress, len);
			readtemp();
	data[0]=tempa1;
	data[1]=tempa2;
    currentAddress += len;
    bytesRemaining -= len;
    return len;
}

uchar   usbFunctionWrite(uchar *data, uchar len)
{
    if(bytesRemaining == 0)
        return 1;               /* done */
    if(len > bytesRemaining)
        len = bytesRemaining;
    eeprom_write_block(data, (uchar *)0 + currentAddress, len);
    currentAddress += len;
    bytesRemaining -= len;
    return bytesRemaining == 0; /* return 1, if done */
}

/* ------------------------------------------------------------------------- */

usbMsgLen_t usbFunctionSetup(uchar data[8])
{
usbRequest_t    *rq = (void *)data;

    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){    /* HID class */
        if(rq->bRequest == USBRQ_HID_GET_REPORT){  /* wValue: ReportType (highbyte), ReportID (lowbyte) */
            bytesRemaining = 8;
            currentAddress = 0;
            return USB_NO_MSG;  
        }else if(rq->bRequest == USBRQ_HID_SET_REPORT){
            bytesRemaining = 8;
            currentAddress = 0;
            return USB_NO_MSG;  
        }
    }else{
        /* none */
    }
    return 0;
}




/* ------------------------------------------------------------------------- */

int main(void)
{
uchar   i;
	DDRB |= 1 << 3;   /* output for LED */
	ledof(); //LED OFF

    wdt_enable(WDTO_1S);
	sensors_init();
    usbInit();
    usbDeviceDisconnect();  /* force enum */
    i = 0;
    while(--i){             /* 250 ms */
        wdt_reset();
        _delay_ms(1);
    }
    usbDeviceConnect();
    sei();
    for(;;){ 
        wdt_reset();
        usbPoll();
		ledof();
   }
    return 0;
}

/* ------------------------------------------------------------------------- */
