#include <stdlib.h>
#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>  /*  sei() */
#include <util/delay.h>     /*  _delay_ms() */
#include <avr/eeprom.h>

#include <avr/pgmspace.h>   /*   usbdrv.h */
#include "usbdrv/usbdrv.h"
#include "usbdrv/oddebug.h"        /*        */

#include "lcd.h"
#include "i2c.c"
#include "lm75.c"
#define LM75_ADDR       (LM75_ADDR_BASE + 0) //pasitikrinti



/* ------------------------------------------------------------------------- */

/* -----------------------------  USB ----------------------------- */
/* ------------------------------------------------------------------------- */

PROGMEM char usbHidReportDescriptor[22] = {    /*   USB */
    0x06, 0x00, 0xff,              // USAGE_PAGE (Generic Desktop)
    0x09, 0x01,                    // USAGE (Vendor Usage 1)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x26, 0xff, 0x00,              //   LOGICAL_MAXIMUM (255)
    0x75, 0x08,                    //   REPORT_SIZE (8)
    0x95, 0x80,                    //   REPORT_COUNT (128)
    0x09, 0x00,                    //   USAGE (Undefined)
    0xb2, 0x02, 0x01,              //   FEATURE (Data,Var,Abs,Buf)
    0xc0                           // END_COLLECTION
};
/*      feature-,     
 *  report-ID (      ).     128
 *  opaque  .
 */

/*        */
static uchar    currentAddress;
static uchar    bytesRemaining;
static uchar	tempa1;
static uchar	tempa2;







int sensors_init(void)
{
	i2c_init();
	return lm75_configure(LM75_ADDR, 0x60); //cia komanda FM75, daugiau tikslumo. Skaityti FM75/LM75 datasheeta.
}

void readtemp(void)
{
    char buffer[16];
    unsigned char dst[2];
	int x=0;
dst[0]=0; dst[1]=0;
		// temperaturos nuskaitymas is FM75 cipo 
			lcd_gotoxy(0,1);
			x = lm75_readRegister(LM75_ADDR, LM75_REG_TEMP, &dst[0], 2);
			if (x==0){
			x=dst[0];
			itoa( x, buffer, 10);
			lcd_puts("temp:         ");
			lcd_gotoxy(5,1);
			lcd_puts(buffer);
			lcd_puts(".");
			x=dst[1]; // Cia skaiciuojam po kablelio. 1/16 laipsnio.
			if (x<1) {lcd_puts("0");}
			if (x<17) {lcd_puts("0");}
			x=dst[1]*4;
			itoa( x, buffer, 10);
			lcd_puts(buffer);
			tempa1=dst[0];
			tempa2=dst[1];
			} else {
			lcd_puts("Sensor error ");
			sensors_init();
			}

}




/* ------------------------------------------------------------------------- */

/* usbFunctionRead() ,       .
 *     .   usbdrv/usbdrv.h.
 */
uchar   usbFunctionRead(uchar *data, uchar len)
{
uchar i;
for(i=0;i<8;i++){ data[i]=i;}
    if(len > bytesRemaining) len = bytesRemaining;
    readtemp();
	
	data[0]=tempa1;
	data[1]=tempa2;
    currentAddress += len;
    bytesRemaining -= len;
    return len;
}

/* usbFunctionWrite() ,       .
 *     .   usbdrv/usbdrv.h.
 */
uchar   usbFunctionWrite(uchar *data, uchar len)
{
    if(bytesRemaining == 0)
        return 1;               /*   */
    if(len > bytesRemaining)
        len = bytesRemaining;
    eeprom_write_block(data, (uchar *)0 + currentAddress, len);
    currentAddress += len;
    bytesRemaining -= len;
    return bytesRemaining == 0; /*  1,      */
}

/* ------------------------------------------------------------------------- */

usbMsgLen_t usbFunctionSetup(uchar data[8])
{
usbRequest_t    *rq = (void *)data;

    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){    /*  HID class */
        if(rq->bRequest == USBRQ_HID_GET_REPORT){  /* wValue: ReportType (highbyte), ReportID (lowbyte) */
            /*       ,    -ID */
            bytesRemaining = 8;
            currentAddress = 0;
            return USB_NO_MSG;  /*  usbFunctionRead()       */
        }else if(rq->bRequest == USBRQ_HID_SET_REPORT){
            /*       ,    -ID */
            bytesRemaining = 8;
            currentAddress = 0;
            return USB_NO_MSG;  /* use usbFunctionWrite()       */
        }
    }else{
        /*    ,       */
    }
    return 0;
}




/* ------------------------------------------------------------------------- */

int main(void)
{
uchar   i;

    wdt_enable(WDTO_1S);
    /*      watchdog,   .   
     *   watchdog (/, )   RESET!
     */
    DBG1(0x00, 0, 0);       /*  :   main */
    /*  RESET:          (pull-up).
     *  ,    D+  D-.  ,      .
     */
	sensors_init();
    lcd_init(LCD_DISP_ON);
	lcd_command(LCD_DISP_ON); 
	lcd_clrscr();
	lcd_puts("FM75 / USB");	
	
	odDebugInit();
    usbInit();
    usbDeviceDisconnect();  /*   ,  ,   ! */
    i = 0;
    while(--i){             /*  USB    > 250  */
        wdt_reset();
        _delay_ms(1);
    }
    usbDeviceConnect();
    sei();
    DBG1(0x01, 0, 0);       /*  :    main */
    for(;;){                /*   main */
        DBG1(0x02, 0, 0);   /*  :   main */
        wdt_reset();
        usbPoll();
		readtemp();
    }
    return 0;
}

/* ------------------------------------------------------------------------- */
